@@ -1,6 +1,7 @@
-from .core import healpix_to_image, image_to_healpix
-from .utils import parse_input_healpix_data, parse_coord_system
 from ..utils import parse_input_data, parse_output_projection
+from ..wcs_utils import has_celestial
+from .core import healpix_to_image, image_to_healpix
+from .utils import parse_coord_system, parse_input_healpix_data
 
 __all__ = ['reproject_from_healpix', 'reproject_to_healpix']
 
@@ -13,7 +14,7 @@ def reproject_from_healpix(input_data, output_projection, shape_out=None,
 
     Parameters
     ----------
-    input_data : str or `~astropy.io.fits.TableHDU` or `~astropy.io.fits.BinTableHDU` or tuple
+    input_data
         The input data to reproject. This can be:
 
             * The name of a HEALPIX FITS file
@@ -80,7 +81,7 @@ def reproject_to_healpix(input_data, coord_system_out, hdu_in=0,
 
     Parameters
     ----------
-    input_data : str or `~astropy.io.fits.HDUList` or `~astropy.io.fits.PrimaryHDU` or `~astropy.io.fits.ImageHDU` or tuple
+    input_data
         The input data to reproject. This can be:
 
             * The name of a FITS file
@@ -125,7 +126,10 @@ def reproject_to_healpix(input_data, coord_system_out, hdu_in=0,
     array_in, wcs_in = parse_input_data(input_data, hdu_in=hdu_in)
     coord_system_out = parse_coord_system(coord_system_out)
 
-    if wcs_in.has_celestial and wcs_in.naxis == 2:
-        return image_to_healpix(array_in, wcs_in, coord_system_out, nside=nside, order=order, nested=nested)
+    if (has_celestial(wcs_in) and wcs_in.low_level_wcs.pixel_n_dim == 2 and
+            wcs_in.low_level_wcs.world_n_dim == 2):
+        return image_to_healpix(array_in, wcs_in, coord_system_out,
+                                nside=nside, order=order, nested=nested)
     else:
-        raise NotImplementedError("Only data with a 2-d celestial WCS can be reprojected to a HEALPIX projection")
+        raise NotImplementedError("Only data with a 2-d celestial WCS can be "
+                                  "reprojected to a HEALPIX projection")
