@@ -1,12 +1,9 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
-import six
+from astropy.utils import deprecated_renamed_argument
 
 from ..utils import parse_input_data, parse_output_projection
-from .core_celestial import _reproject_celestial
-from .core_full import _reproject_full
+from .core import _reproject_full
 
 __all__ = ['reproject_interp']
 
@@ -17,16 +14,16 @@ ORDER['biquadratic'] = 2
 ORDER['bicubic'] = 3
 
 
+@deprecated_renamed_argument('independent_celestial_slices', None, since='0.6')
 def reproject_interp(input_data, output_projection, shape_out=None, hdu_in=0,
-                     order='bilinear', independent_celestial_slices=False,
-                     output_array=None, return_footprint=True):
+                     order='bilinear', output_array=None, return_footprint=True):
     """
     Reproject data to a new projection using interpolation (this is typically
     the fastest way to reproject an image).
 
     Parameters
     ----------
-    input_data : str or `~astropy.io.fits.HDUList` or `~astropy.io.fits.PrimaryHDU` or `~astropy.io.fits.ImageHDU` or tuple
+    input_data
         The input data to reproject. This can be:
 
             * The name of a FITS file
@@ -37,6 +34,8 @@ def reproject_interp(input_data, output_projection, shape_out=None, hdu_in=0,
             * A tuple where the first element is a `~numpy.ndarray` and the
               second element is either a `~astropy.wcs.WCS` or a
               `~astropy.io.fits.Header` object
+            * A `astropy.nddata.NDData` object from which the ``.data`` and
+              ``.wcs`` attributes will be used as the input data.
 
     output_projection : `~astropy.wcs.WCS` or `~astropy.io.fits.Header`
         The output projection, which can be either a `~astropy.wcs.WCS`
@@ -58,26 +57,12 @@ def reproject_interp(input_data, output_projection, shape_out=None, hdu_in=0,
 
         or an integer. A value of ``0`` indicates nearest neighbor
         interpolation.
-    independent_celestial_slices : bool, optional
-        This can be set to ``True`` for n-dimensional input in the following case
-        (all conditions have to be fulfilled):
-
-            * The number of pixels in each non-celestial dimension is the same
-              between the input and target header.
-            * The WCS coordinates along the non-celestial dimensions are the
-              same between the input and target WCS.
-            * The celestial WCS component is independent from other WCS
-              coordinates.
-
-        In this special case, we can make things a little faster by
-        reprojecting each celestial slice independently using the same
-        transformation.
     output_array : None or `~numpy.ndarray`
         An array in which to store the reprojected data.  This can be any numpy
         array including a memory map, which may be helpful when dealing with
         extremely large files.
     return_footprint : bool
-        Return the footprint in addition to the output array?
+        Whether to return the footprint in addition to the output array.
 
     Returns
     -------
@@ -90,14 +75,11 @@ def reproject_interp(input_data, output_projection, shape_out=None, hdu_in=0,
     """
 
     array_in, wcs_in = parse_input_data(input_data, hdu_in=hdu_in)
-    wcs_out, shape_out = parse_output_projection(output_projection, shape_out=shape_out)
+    wcs_out, shape_out = parse_output_projection(output_projection, shape_out=shape_out,
+                                                 output_array=output_array)
 
-    if isinstance(order, six.string_types):
+    if isinstance(order, str):
         order = ORDER[order]
 
-    if (wcs_in.is_celestial and wcs_in.naxis == 2) or independent_celestial_slices:
-        return _reproject_celestial(array_in, wcs_in, wcs_out, shape_out=shape_out, order=order,
-                                    array_out=output_array, return_footprint=return_footprint)
-    else:
-        return _reproject_full(array_in, wcs_in, wcs_out, shape_out=shape_out, order=order,
-                               return_footprint=return_footprint)
+    return _reproject_full(array_in, wcs_in, wcs_out, shape_out=shape_out, order=order,
+                           array_out=output_array, return_footprint=return_footprint)
