@@ -1,36 +1,32 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 import os
 import itertools
 
 import numpy as np
-from astropy.io import fits
-from astropy.wcs import WCS
-from astropy.utils.data import get_pkg_data_filename
 import pytest
+from astropy import units as u
+from astropy.io import fits
+from astropy.utils.data import get_pkg_data_filename
+from astropy.wcs import WCS
+from astropy.wcs.wcsapi import HighLevelWCSWrapper, SlicedLowLevelWCS
+from numpy.testing import assert_allclose
 
-from ..core_celestial import _reproject_celestial
-from ..core_full import _reproject_full
 from ..high_level import reproject_interp
+from ...tests.helpers import array_footprint_to_hdulist
 
 # TODO: add reference comparisons
 
-
 DATA = os.path.join(os.path.dirname(__file__), '..', '..', 'tests', 'data')
 
 
-def array_footprint_to_hdulist(array, footprint, header):
-    hdulist = fits.HDUList()
-    hdulist.append(fits.PrimaryHDU(array, header))
-    hdulist.append(fits.ImageHDU(footprint, header, name='footprint'))
-    return hdulist
+def as_high_level_wcs(wcs):
+    return HighLevelWCSWrapper(SlicedLowLevelWCS(wcs, Ellipsis))
 
 
-@pytest.mark.array_compare()
-def test_reproject_celestial_2d_gal2equ():
+@pytest.mark.array_compare(single_reference=True)
+@pytest.mark.parametrize('wcsapi', (False, True))
+def test_reproject_celestial_2d_gal2equ(wcsapi):
     """
     Test reprojection of a 2D celestial image, which includes a coordinate
     system conversion.
@@ -41,21 +37,31 @@ def test_reproject_celestial_2d_gal2equ():
     header_out['CTYPE2'] = 'DEC--TAN'
     header_out['CRVAL1'] = 266.39311
     header_out['CRVAL2'] = -28.939779
-    array_out, footprint_out = reproject_interp(hdu_in, header_out)
+
+    if wcsapi:  # Enforce a pure wcsapi API
+        wcs_in, data_in = as_high_level_wcs(WCS(hdu_in.header)), hdu_in.data
+        wcs_out = as_high_level_wcs(WCS(header_out))
+        shape_out = header_out['NAXIS2'], header_out['NAXIS1']
+        array_out, footprint_out = reproject_interp((data_in, wcs_in),
+                                                    wcs_out, shape_out=shape_out)
+    else:
+        array_out, footprint_out = reproject_interp(hdu_in, header_out)
+
     return array_footprint_to_hdulist(array_out, footprint_out, header_out)
 
 
 # Note that we can't use independent_celestial_slices=True and reorder the
 # axes, hence why we need to prepare the combinations in this way.
 AXIS_ORDER = list(itertools.permutations((0, 1, 2)))
-COMBINATIONS = [(True, (0, 1, 2))]
-for axis_order in AXIS_ORDER:
-    COMBINATIONS.append((False, axis_order))
+COMBINATIONS = []
+for wcsapi in (False, True):
+    for axis_order in AXIS_ORDER:
+        COMBINATIONS.append((wcsapi, axis_order))
 
 
 @pytest.mark.array_compare(single_reference=True)
-@pytest.mark.parametrize(('indep_slices', 'axis_order'), tuple(COMBINATIONS))
-def test_reproject_celestial_3d_equ2gal(indep_slices, axis_order):
+@pytest.mark.parametrize(('wcsapi', 'axis_order'), tuple(COMBINATIONS))
+def test_reproject_celestial_3d_equ2gal(wcsapi, axis_order):
     """
     Test reprojection of a 3D cube with celestial components, which includes a
     coordinate system conversion (the original header is in equatorial
@@ -87,13 +93,21 @@ def test_reproject_celestial_3d_equ2gal(indep_slices, axis_order):
         hdu_in.header = wcs_in.to_header()
         hdu_in.data = np.transpose(hdu_in.data, axis_order)
 
-    array_out, footprint_out = reproject_interp(hdu_in, header_out,
-                                                independent_celestial_slices=indep_slices)
+    if wcsapi:  # Enforce a pure wcsapi API
+        wcs_in, data_in = as_high_level_wcs(WCS(hdu_in.header)), hdu_in.data
+        wcs_out = as_high_level_wcs(WCS(header_out))
+        shape_out = header_out['NAXIS3'], header_out['NAXIS2'], header_out['NAXIS1']
+        array_out, footprint_out = reproject_interp((data_in, wcs_in),
+                                                    wcs_out, shape_out=shape_out)
+    else:
+        array_out, footprint_out = reproject_interp(hdu_in, header_out)
+
     return array_footprint_to_hdulist(array_out, footprint_out, header_out)
 
 
-@pytest.mark.array_compare()
-def test_small_cutout():
+@pytest.mark.array_compare(single_reference=True)
+@pytest.mark.parametrize('wcsapi', (False, True))
+def test_small_cutout(wcsapi):
     """
     Test reprojection of a cutout from a larger image (makes sure that the
     pre-reprojection cropping works)
@@ -108,7 +122,16 @@ def test_small_cutout():
     header_out['CRVAL2'] = -28.939779
     header_out['CRPIX1'] = 5.1
     header_out['CRPIX2'] = 4.7
-    array_out, footprint_out = reproject_interp(hdu_in, header_out)
+
+    if wcsapi:  # Enforce a pure wcsapi API
+        wcs_in, data_in = as_high_level_wcs(WCS(hdu_in.header)), hdu_in.data
+        wcs_out = as_high_level_wcs(WCS(header_out))
+        shape_out = header_out['NAXIS2'], header_out['NAXIS1']
+        array_out, footprint_out = reproject_interp((data_in, wcs_in),
+                                                    wcs_out, shape_out=shape_out)
+    else:
+        array_out, footprint_out = reproject_interp(hdu_in, header_out)
+
     return array_footprint_to_hdulist(array_out, footprint_out, header_out)
 
 
@@ -226,7 +249,8 @@ def test_spectral_mismatch_3d():
 
     with pytest.raises(ValueError) as exc:
         array_out, footprint_out = reproject_interp((data, wcs1), wcs2, shape_out=(1, 2, 3))
-    assert exc.value.args[0] == "The input (VOPT) and output (FREQ) spectral coordinate types are not equivalent."
+    assert exc.value.args[0] == ("The input (VOPT) and output (FREQ) spectral "
+                                 "coordinate types are not equivalent.")
 
     header_out['CTYPE3'] = 'BANANAS'
     wcs2 = WCS(header_out)
@@ -275,7 +299,7 @@ def test_slice_reprojection():
     wcs_in = WCS(header_in)
     wcs_out = WCS(header_out)
 
-    out_cube, out_cube_valid = _reproject_full(inp_cube, wcs_in, wcs_out, (2, 4, 5))
+    out_cube, out_cube_valid = reproject_interp((inp_cube, wcs_in), wcs_out, shape_out=(2, 4, 5))
 
     # we expect to be projecting from
     # inp_cube = np.arange(3, dtype='float').repeat(4*5).reshape(3,4,5)
@@ -283,7 +307,8 @@ def test_slice_reprojection():
     # inp_cube_interp = (inp_cube[:-1]+inp_cube[1:])/2.
     # which is confirmed by
     # map_coordinates(inp_cube.astype('float'), new_coords, order=1, cval=np.nan, mode='constant')
-    # np.testing.assert_allclose(inp_cube_interp, map_coordinates(inp_cube.astype('float'), new_coords, order=1, cval=np.nan, mode='constant'))
+    # np.testing.assert_allclose(inp_cube_interp, map_coordinates(inp_cube.astype('float'),
+    # new_coords, order=1, cval=np.nan, mode='constant'))
 
     assert out_cube.shape == (2, 4, 5)
     assert out_cube_valid.sum() == 40.
@@ -327,7 +352,8 @@ def test_inequal_wcs_dims():
     wcs_out = WCS(header_out)
 
     with pytest.raises(ValueError) as ex:
-        out_cube, out_cube_valid = reproject_interp((inp_cube, header_in), wcs_out, shape_out=(2, 4, 5))
+        out_cube, out_cube_valid = reproject_interp((inp_cube, header_in),
+                                                    wcs_out, shape_out=(2, 4, 5))
     assert str(ex.value) == "Output WCS has a spectral component but input WCS does not"
 
 
@@ -342,11 +368,11 @@ def test_different_wcs_types():
     header_in['CTYPE3'] = 'VELO'
     header_in['CUNIT3'] = 'm/s'
 
-    wcs_in = WCS(header_in)
     wcs_out = WCS(header_out)
 
     with pytest.raises(ValueError) as ex:
-        out_cube, out_cube_valid = reproject_interp((inp_cube, header_in), wcs_out, shape_out=(2, 4, 5))
+        out_cube, out_cube_valid = reproject_interp((inp_cube, header_in),
+                                                    wcs_out, shape_out=(2, 4, 5))
     assert str(ex.value) == ("The input (VELO) and output (VRAD) spectral "
                              "coordinate types are not equivalent.")
 
@@ -389,7 +415,7 @@ def test_reproject_3d_celestial_correctness_ra2gal():
     np.testing.assert_allclose(out_cube[:, 0, 0], ((inp_cube[:-1] + inp_cube[1:]) / 2.)[:, 0, 0])
 
 
-def test_reproject_celestial_3d():
+def test_reproject_with_output_array():
     """
     Test both full_reproject and slicewise reprojection. We use a case where the
     non-celestial slices are the same and therefore where both algorithms can
@@ -399,40 +425,8 @@ def test_reproject_celestial_3d():
     header_in = fits.Header.fromtextfile(get_pkg_data_filename('../../tests/data/cube.hdr'))
 
     array_in = np.ones((3, 200, 180))
-
-    # TODO: here we can check that if we change the order of the dimensions in
-    # the WCS, things still work properly
-
-    wcs_in = WCS(header_in)
-    wcs_out = wcs_in.deepcopy()
-    wcs_out.wcs.ctype = ['GLON-SIN', 'GLAT-SIN', wcs_in.wcs.ctype[2]]
-    wcs_out.wcs.crval = [158.0501, -21.530282, wcs_in.wcs.crval[2]]
-    wcs_out.wcs.crpix = [50., 50., wcs_in.wcs.crpix[2] + 0.4]
-
-    out_full, foot_full = _reproject_full(array_in, wcs_in, wcs_out, (3, 160, 170))
-
-    out_celestial, foot_celestial = _reproject_celestial(array_in, wcs_in, wcs_out, (3, 160, 170))
-
-    np.testing.assert_allclose(out_full, out_celestial)
-    np.testing.assert_allclose(foot_full, foot_celestial)
-
-
-def test_reproject_celestial_3d_withoutputarray():
-    """
-    Test both full_reproject and slicewise reprojection. We use a case where the
-    non-celestial slices are the same and therefore where both algorithms can
-    work.
-    """
-
-    header_in = fits.Header.fromtextfile(get_pkg_data_filename('../../tests/data/cube.hdr'))
-
-    array_in = np.ones((3, 200, 180))
-    outshape = (3, 160, 170)
-    out_full = np.empty(outshape)
-    out_celestial = np.empty(outshape)
-
-    # TODO: here we can check that if we change the order of the dimensions in
-    # the WCS, things still work properly
+    shape_out = (3, 160, 170)
+    out_full = np.empty(shape_out)
 
     wcs_in = WCS(header_in)
     wcs_out = wcs_in.deepcopy()
@@ -441,12 +435,75 @@ def test_reproject_celestial_3d_withoutputarray():
     wcs_out.wcs.crpix = [50., 50., wcs_in.wcs.crpix[2] + 0.4]
 
     # TODO when someone learns how to do it: make sure the memory isn't duplicated...
-    _ = _reproject_full(array_in, wcs_in, wcs_out, shape_out=outshape,
-                        array_out=out_full, return_footprint=False)
-    assert out_full is _
+    returned_array = reproject_interp((array_in, wcs_in), wcs_out,
+                                      output_array=out_full, return_footprint=False)
 
-    _ = _reproject_celestial(array_in, wcs_in, wcs_out, shape_out=outshape,
-                             array_out=out_celestial, return_footprint=False)
-    assert out_celestial is _
+    assert out_full is returned_array
 
-    np.testing.assert_allclose(out_full, out_celestial)
+
+@pytest.mark.array_compare(single_reference=True)
+@pytest.mark.parametrize('file_format', ['fits', 'asdf'])
+def test_reproject_roundtrip(file_format):
+
+    # Test the reprojection with solar data, which ensures that the masking of
+    # pixels based on round-tripping works correctly. Using asdf is not just
+    # about testing a different format but making sure that GWCS works.
+
+    pytest.importorskip('sunpy', minversion='1.0.4')
+    from sunpy.map import Map
+    from sunpy.coordinates.ephemeris import get_body_heliographic_stonyhurst
+
+    if file_format == 'fits':
+        map_aia = Map(os.path.join(DATA, 'aia_171_level1.fits'))
+        data = map_aia.data
+        wcs = map_aia.wcs
+        date = map_aia.date
+        target_wcs = wcs.deepcopy()
+    elif file_format == 'asdf':
+        pytest.importorskip('astropy', minversion='4.0')
+        pytest.importorskip('gwcs', minversion='0.12')
+        asdf = pytest.importorskip('asdf')
+        aia = asdf.open(os.path.join(DATA, 'aia_171_level1.asdf'))
+        data = aia['data'][...]
+        wcs = aia['wcs']
+        date = wcs.output_frame.reference_frame.obstime
+        target_wcs = Map(os.path.join(DATA, 'aia_171_level1.fits')).wcs.deepcopy()
+    else:
+        raise ValueError('file_format should be fits or asdf')
+
+    # Reproject to an observer on Venus
+
+    target_wcs.wcs.cdelt = ([24, 24]*u.arcsec).to(u.deg)
+    target_wcs.wcs.crpix = [64, 64]
+    venus = get_body_heliographic_stonyhurst('venus', date)
+    target_wcs.heliographic_observer = venus
+
+    output, footprint = reproject_interp((data, wcs), target_wcs, (128, 128))
+
+    return array_footprint_to_hdulist(output, footprint, target_wcs.to_header())
+
+
+def test_identity_with_offset():
+
+    # Reproject an array and WCS to itself but with a margin, which should
+    # end up empty. This is a regression test for a bug that caused some
+    # values to extend beyond the original footprint.
+
+    wcs = WCS(naxis=2)
+    wcs.wcs.ctype = 'RA---TAN', 'DEC--TAN'
+    wcs.wcs.crpix = 322, 151
+    wcs.wcs.crval = 43, 23
+    wcs.wcs.cdelt = -0.1, 0.1
+    wcs.wcs.equinox = 2000.
+
+    array_in = np.random.random((233, 123))
+
+    wcs_out = wcs.deepcopy()
+    wcs_out.wcs.crpix += 1
+    shape_out = (array_in.shape[0] + 2, array_in.shape[1] + 2)
+
+    array_out, footprint = reproject_interp((array_in, wcs), wcs_out, shape_out=shape_out)
+
+    expected = np.pad(array_in, 1, 'constant', constant_values=np.nan)
+
+    assert_allclose(expected, array_out, atol=1e-10)
