@@ -2,4 +2,4 @@
 """
 Routines to compute pixel overlap areas.
 """
-from .high_level import *
+from .high_level import *  # noqa
