@@ -1,22 +1,21 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 from ..utils import parse_input_data, parse_output_projection
+from ..wcs_utils import has_celestial
 from .core import _reproject_celestial
 
 __all__ = ['reproject_exact']
 
 
 def reproject_exact(input_data, output_projection, shape_out=None, hdu_in=0,
-                    parallel=True):
+                    parallel=True, return_footprint=True):
     """
     Reproject data to a new projection using flux-conserving spherical
     polygon intersection (this is the slowest algorithm).
 
     Parameters
     ----------
-    input_data : str or `~astropy.io.fits.HDUList` or `~astropy.io.fits.PrimaryHDU` or `~astropy.io.fits.ImageHDU` or tuple
+    input_data
         The input data to reproject. This can be:
 
             * The name of a FITS file
@@ -27,6 +26,8 @@ def reproject_exact(input_data, output_projection, shape_out=None, hdu_in=0,
             * A tuple where the first element is a `~numpy.ndarray` and the
               second element is either a `~astropy.wcs.WCS` or a
               `~astropy.io.fits.Header` object
+            * A `astropy.nddata.NDData` object from which the ``.data`` and
+              ``.wcs`` attributes will be used as the input data.
 
     output_projection : `~astropy.wcs.WCS` or `~astropy.io.fits.Header`
         The output projection, which can be either a `~astropy.wcs.WCS`
@@ -43,6 +44,8 @@ def reproject_exact(input_data, output_projection, shape_out=None, hdu_in=0,
         the number of logical CPUs detected on the machine. If ``False``, a
         serial implementation is chosen. If the flag is a positive integer ``n``
         greater than one, a parallel implementation using ``n`` processes is chosen.
+    return_footprint : bool
+        Whether to return the footprint in addition to the output array.
 
     Returns
     -------
@@ -57,7 +60,9 @@ def reproject_exact(input_data, output_projection, shape_out=None, hdu_in=0,
     array_in, wcs_in = parse_input_data(input_data, hdu_in=hdu_in)
     wcs_out, shape_out = parse_output_projection(output_projection, shape_out=shape_out)
 
-    if wcs_in.has_celestial and wcs_in.naxis == 2:
-        return _reproject_celestial(array_in, wcs_in, wcs_out, shape_out=shape_out, parallel=parallel)
+    if has_celestial(wcs_in) and wcs_in.pixel_n_dim == 2 and wcs_in.world_n_dim == 2:
+        return _reproject_celestial(array_in, wcs_in, wcs_out, shape_out=shape_out,
+                                    parallel=parallel, return_footprint=return_footprint)
     else:
-        raise NotImplementedError("Currently only data with a 2-d celestial WCS can be reprojected using flux-conserving algorithm")
+        raise NotImplementedError("Currently only data with a 2-d celestial "
+                                  "WCS can be reprojected using flux-conserving algorithm")
