@@ -58,6 +58,7 @@ void SaveSharedSeg(Vec *p, Vec *q);
 void PrintPolygon();
 
 void ComputeIntersection(Vec *P, Vec *Q);
+void EnsureCounterClockWise(Vec *V);
 
 int UpdateInteriorFlag(Vec *p, int interiorFlag, int pEndpointFromQdir,
                        int qEndpointFromPdir);
@@ -122,11 +123,50 @@ double computeOverlap(double *ilon, double *ilat, double *olon, double *olat,
     Q[i].z = sin(olat[i]);
   }
 
+  EnsureCounterClockWise(P);
+  EnsureCounterClockWise(Q);
+
   ComputeIntersection(P, Q);
 
   return (Girard());
 }
 
+void EnsureCounterClockWise(Vec *V) {
+
+  // Make sure that the polygon is counter-clockwise. For now we assume that we
+  // are dealing with convex quadrilaterals, and therefore we can just check
+  // the two first sides of the polygon.
+
+  Vec S1, S2, C;
+  double dir;
+  double tmp;
+
+  S1.x = V[1].x - V[0].x;
+  S1.y = V[1].y - V[0].y;
+  S1.z = V[1].z - V[0].z;
+
+  S2.x = V[2].x - V[1].x;
+  S2.y = V[2].y - V[1].y;
+  S2.z = V[2].z - V[1].z;
+
+  Cross(&S1, &S2, &C);
+
+  dir = Dot(&V[1], &C);
+
+  if (dir < 0) {
+    tmp = V[2].x;
+    V[2].x = V[0].x;
+    V[0].x = tmp;
+    tmp = V[2].y;
+    V[2].y = V[0].y;
+    V[0].y = tmp;
+    tmp = V[2].z;
+    V[2].z = V[0].z;
+    V[0].z = tmp;
+  }
+
+}
+
 /*
  * Find the polygon defining the area of overlap
  * between the two input polygons P and Q.
@@ -797,7 +837,7 @@ int Cross(Vec *v1, Vec *v2, Vec *v3) {
   v3->y = -v1->x * v2->z + v2->x * v1->z;
   v3->z = v1->x * v2->y - v2->x * v1->y;
 
-  if (v3->x == 0. && v3->y == 0. && v3->z == 0.)
+  if (fabs(v3->x) < 1.e-18 && fabs(v3->y) < 1.e-18 && fabs(v3->z) < 1.e-18)
     return 0;
 
   return 1;
@@ -876,7 +916,43 @@ double Girard() {
 
   for (i = 0; i < nv; ++i) {
     Cross(&V[i], &V[(i + 1) % nv], &side[i]);
+  }
 
+  // De-duplicate vertices that are extremely close to each other otherwise
+  // the angles determined in the next steps are not accurate. Need to loop
+  // backwards to avoid affecting future sides that need to be checked.
+
+  for (i = nv - 1; i >= 0; --i) {
+
+    // We don't use TOLERANCE here since it is too large for our purposes here
+
+    if (Dot(&side[i], &side[i]) < 1e-30) {
+
+      if (DEBUG >= 4) {
+        printf("Girard(): ---------- Corner %d duplicate; "
+               "Remove point %d -------------\n",
+               i, i);
+        fflush(stdout);
+      }
+
+      --nv;
+
+      for (j = i; j < nv; ++j) {
+        V[j].x = V[j + 1].x;
+        V[j].y = V[j + 1].y;
+        V[j].z = V[j + 1].z;
+        side[j].x = side[j + 1].x;
+        side[j].y = side[j + 1].y;
+        side[j].z = side[j + 1].z;
+      }
+
+    }
+  }
+
+  if (nv < 3)
+    return 0;
+
+  for (i = 0; i < nv; ++i) {
     Normalize(&side[i]);
   }
 
